<?php

namespace Drupal\migrate_media_handler;

use Drupal\Core\Config\ConfigFactory;
use Drupal\Core\Entity\EntityTypeManager;
use Drupal\Core\DependencyInjection\ContainerInjectionInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * MediaMaker takes file paths and makes file & media entities if possible.
 *
 * Requires the presence of a text field on all destination media entities
 * named `field_original_ref`. This allows for lookup by file hash to avoid
 * duplication. This field can be deleted after migration is complete.
 */
class MediaMaker implements ContainerInjectionInterface {

  /**
   * Entity Type Manager service.
   *
   * @var \Drupal\Core\Entity\EntityTypeManager
   */
  protected $entityManager;

  /**
   * ConfigFactory service.
   *
   * @var \Drupal\Core\Config\ConfigFactory
   */
  protected $configFactory;

  /**
   * Source variable.
   *
   * @var string
   */
  protected $source = '';

  /**
   * Destination variable.
   *
   * @var string
   */
  protected $destination = '';

  /**
   * File Owner variable.
   *
   * @var string
   */
  protected $owner = '';

  /**
   * Site URI Regex variable.
   *
   * @var string
   */
  protected $siteUri = '';

  /**
   * {@inheritdoc}
   */
  public function __construct(EntityTypeManager $entity_manager, configFactory $config_factory) {
    $this->entityManager = $entity_manager;
    $this->configFactory = $config_factory;

    // Pull media setting from config. Can be overridden if necessary.
    $config = $this->configFactory->get('migrate_media_handler.settings');
    $this->source = $config->get('file_source');
    $this->destination = $config->get('file_dest');
    $this->owner = $config->get('file_owner');
    $this->siteUri = $config->get('site_uri');
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('entity_type.manager'),
      $container->get('config.factory')
    );
  }

  /**
   * Find existing media by file hash.
   *
   * @param string $hash
   *   File hash to lookup in field_original_ref.
   *
   * @return bool|\Drupal\media\Entity\Media
   *   Return media entity or FALSE
   */
  public function findExistingMediaByHash(string $hash) {
    $media = FALSE;
    // Lookup the media entity by hash of file data in field_original_ref.
    $query = $this->entityManager->getStorage('media')->getQuery()
      ->condition('status', 1)
      ->condition('field_original_ref', $hash, '=');
    $mids = $query->execute();
    // If found, `mid` is the reference for the content.
    if (!empty($mids)) {
      $mid = array_pop($mids);
      // This check is to make sure that the media /actually/ exists.
      $media = $this->entityManager->getStorage('media')->load($mid);
    }
    return $media;
  }

  /**
   * Find existing media by file path, using the hash.
   *
   * @param string $path
   *   Path from which to generate hash for comparison.
   *
   * @return bool|\Drupal\media\Entity\Media
   *   Return media entity or FALSE
   */
  public function findExistingMediaByPath(string $path) {
    // Hash file, then find by stored hash.
    $hash = sha1_file($path);
    return $this->findExistingMediaByHash($hash);
  }

  /**
   * Make a file entity out of a file path. Move file to current file directory.
   *
   * @param string $path
   *   File path of existing legacy file.
   *
   * @return bool|\Drupal\file\Entity\File
   *   File entity or FALSE
   */
  public function makeFileEntity(string $path) {
    $destination = str_ireplace($this->source, $this->destination, $path);
    $filename = pathinfo($path, PATHINFO_BASENAME);

    if (file_exists($path)) {
      // Create file entity.
      $file = $this->entityManager->getStorage('file')->create();
      $file->setFileUri($destination);
      $file->setFilename($filename);
      $file->setMimeType(mime_content_type($path));
      $file->setSize(filesize($path));
      $file->setOwnerId($this->owner);
      $file->setPermanent();
      $file->save();

      // Copy file to permanent destination.
      $dir = dirname($destination);
      if (!file_exists($dir)) {
        mkdir($dir, 0770, TRUE);
      }
      file_put_contents($destination, file_get_contents($path));
      $file->save();
      return $file;
    }
    return FALSE;
  }

  /**
   * Make an image media entity out of a file. Add alt, title if avail.
   *
   * @param string $file_id
   *   ID of existing file entity.
   * @param string $hash
   *   File hash of existing legacy file.
   * @param string $alt
   *   Alt Text. Please supply if at all possible!
   * @param string $title
   *   Image title. Please supply if at all possible!
   *
   * @return bool|\Drupal\media\Entity\Media
   *   Return media entity or FALSE
   */
  public function makeImageEntity($file_id, $hash = '', $alt = '', $title = '') {
    $media = FALSE;
    // Load file entity.
    $file = $this->entityManager->getStorage('file')->load($file_id);
    // If that's successful, carry on.
    if ($file) {
      // If no file hash was specified, hash it.
      if (!$hash) {
        $hash = sha1_file($file->getFileUri());
      }
      // Lookup media entity by file hash.
      $media = $this->findExistingMediaByHash($hash);

      // If it wasn't found, make it!
      if (!$media) {
        // Create media entity with saved file.
        // Please note accessibility concerns around empty alt & title.
        $media = $this->entityManager->getStorage('media')->create([
          'bundle' => 'image',
          'field_original_ref' => $hash,
          'field_media_image' => [
            'target_id' => $file_id,
            'alt' => $alt,
            'title' => $title,
          ],
          'langcode' => 'en',
        ]);
        $owner = $file->getOwnerId();
        $filename = $file->getFilename();
        $media->setOwnerId($owner);
        $media->setName($filename);
        $media->save();
      }
    }
    return $media;
  }

  /**
   * Make a document media entity out of a file.
   *
   * @param string $file_id
   *   ID of existing file entity.
   * @param string $hash
   *   File hash of existing legacy file.
   *
   * @return bool|\Drupal\media\Entity\Media
   *   Return media entity or FALSE
   */
  public function makeDocumentEntity($file_id, $hash = '') {
    $media = FALSE;
    // Load file entity.
    $file = $this->entityManager->getStorage('file')->load($file_id);
    // If that's successful, carry on.
    if ($file) {
      // If no file hash was specified, hash it.
      if (!$hash) {
        $hash = sha1_file($file->getFileUri());
      }
      // Lookup media entity by file hash.
      $media = $this->findExistingMediaByHash($hash);

      // If it wasn't found, make it!
      if (!$media) {
        // Create media entity with saved file.
        $media = $this->entityManager->getStorage('media')->create([
          'bundle' => 'document',
          'field_original_ref' => $hash,
          'field_media_image' => [
            'target_id' => $file_id,
          ],
          'langcode' => 'en',
        ]);
        $owner = $file->getOwnerId();
        $filename = $file->getFilename();
        $media->setOwnerId($owner);
        $media->setName($filename);
        $media->save();
      }
    }
    return $media;
  }

  /**
   * Utility function to get source file repository filepath.
   *
   * @param string $path
   *   Path from which to generate hash for comparison.
   */
  public function getSourceFilePath($path = '') {
    // If link is full-path (contains http:// or https://).
    if (preg_match('/https?:\/\//', $path) !== 0) {
      // If link is full-path to existing site.
      if (preg_match($this->siteUri, $path) !== 0) {
        // Swap out full path for public://, and remove sites/default/files.
        $file_path = preg_replace($this->siteUri, $this->source, $path);
        $file_path = str_ireplace("/sites/default/files", "", $file_path);
      }
      // Otherwise, it's an external link.
      else {
        $file_path = $path;
      }
    }
    // Otherwise, it's just a relative link.
    else {
      $file_path = $path;
    }

    return $file_path;
  }

}
